###############################################################################
# Licensed Materials - Property of IBM
# (c) Copyright IBM Corporation 2013, 2014. All Rights Reserved.
#
# Note to U.S. Government Users Restricted Rights:  
# Use, duplication or disclosure restricted by GSA ADP Schedule 
# Contract with IBM Corp. 
###############################################################################

#
# Script that uninstalls the Rational Integration Tester JDBC provider from WebSphere Application Server
#

import sys

#######################
#                     #
# Parameter checking  #
#                     #
#######################

def usage():
  sys.exit("\nUsage: wsadmin.bat -f uninstallJDBCProvider.py\n" +
           "       | cell\n" +
           "       | node <node name>\n" +
           "       | node <node name> server <server name>\n" +
           "       | cluster <cluster name>\n\n" +
           "With no other arguments the driver will be removed from every " +
           "server, node, and cluster in the WebSphere Application Server cell. " + 
           "To limit the removal to the deployment manager, a single node, " +
           "a single server, or a cluster, provide either 'cell', or a node name, " +
           "or node and server names, or a cluster name.\n")
#endDef

providerName = "Green Hat JDBC Provider"
driverVariableName = "GH_DRIVER_PATH"
errors = 0
completeUninstall = 0

#
# Parse the command line to find the correct scope to remove the driver from
#
def findScope(args):
  global completeUninstall
  argc = len(args)
  if argc == 3 or argc > 4:
    usage()
  
  scopes = []
  if argc == 0:
    # Remove from the cell, all nodes, all servers and all clusters
    completeUninstall = 1
    scopes.append("/Cell:" + java.lang.System.getenv('WAS_CELL') + "/")
    nodes = AdminConfig.list("Node").splitlines()
    for node in nodes:
      nodeName = AdminConfig.showAttribute(node, "name")
      scopes.append("/Node:" + nodeName + "/")
      servers = AdminConfig.list("Server", node).splitlines()
      for server in servers:
        scopes.append("/Node:" + nodeName +
                      "/Server:"+ AdminConfig.showAttribute(server, "name") + "/")
    clusters = AdminConfig.list("ServerCluster").splitlines()
    for cluster in clusters:
      scopes.append("/ServerCluster:" + AdminConfig.showAttribute(cluster, "name") + "/")
  elif argc == 1:
    if args[0] == "cell":
      scopes.append("/Cell:" + java.lang.System.getenv('WAS_CELL') + "/")
    else:
      usage()
  elif argc == 2:
    if args[0] == "node":
      nodeName = args[1]
      scopes.append("/Node:" + nodeName + "/")
      node = AdminConfig.getid("/Node:" + nodeName + "/")
      servers = AdminConfig.list("Server", node).splitlines()
      for server in servers:
        serverName = AdminConfig.showAttribute(server, "name")
        scopes.append("/Node:" + nodeName + "/Server:" + serverName + "/");
    elif args[0] == "cluster":
      scopes.append("/ServerCluster:" + args[1] + "/")
      cluster = AdminConfig.getid("/ServerCluster:" + args[1] + "/");
      clusterMembers = AdminConfig.list("ClusterMember", cluster).splitlines()
      for clusterMember in clusterMembers:
        nodeName = AdminConfig.showAttribute(clusterMember, "nodeName")
        serverName = AdminConfig.showAttribute(clusterMember, "memberName")
        scopes.append("/Node:" + nodeName + "/Server:" + serverName + "/")
    else:
      usage()
  elif argc == 4:
    if args[0] == "node" and args[2] == "server":
      scopes.append("/Node:" + args[1] + "/Server:" + args[3] + "/")
    else:
      usage()
  return scopes
#endDef

#
# Search for the provider at the given scope, and if it is there, remove it
# scope : WAS scope, such as "/Cell:<cellName>/" or "/ServerCluster:<clusterName>/"
#
def removeProvider(scope):
  global errors
  print "Checking for JDBC provider at scope " + scope
  provider = AdminConfig.getid(scope + "JDBCProvider:"+ providerName + "/")
  if provider:
    scopeId = AdminConfig.getid(scope)
    dataSources = AdminConfig.list("DataSource",scopeId).splitlines()
    for dataSource in dataSources:
      dataSourceProvider = AdminConfig.showAttribute(dataSource, 'provider')
      if dataSourceProvider == provider:
        print("\nERROR: Unable to remove existing Green Hat JDBC Provider as it is in use. " +
            "Please use Rational Integration Tester to disable JDBC virtualization "+
            "for all data sources at " + scope + " scope and retry.")
        errors = errors+1
        return
    #
    # If we got to here then we have a provider that it is safe for us to remove
    #
    print("  Removing JDBC provider")
    AdminConfig.remove(provider)
#endDef

###########################
# Main part of the script #
###########################

#
# Detect if there is a dmgr controlling the cell
#
try:
  connectedNode = AdminControl.getNode()
except:
  print("\nERROR: Please start the WebSphere processes before running this script.\n")
  raise

connectedServer = AdminControl.queryNames("node="+connectedNode+",type=Server,*")
dmgr = AdminControl.getAttribute(connectedServer, 'processType') == "DeploymentManager"

scopes = findScope(sys.argv)
for scope in scopes:
  removeProvider(scope)

#
# If this is a complete uninstall, and there were no errors, then it is safe to clean
# up the classpath variables as well.
#
if completeUninstall == 1 and errors == 0:
  print "\nRemoving JDBC Provider classpath variables"
  variableMaps = AdminConfig.list("VariableMap").splitlines()
  for variableMap in variableMaps:
    variables = AdminConfig.list("VariableSubstitutionEntry",variableMap).splitlines()
    for variable in variables:
      name = AdminConfig.showAttribute(variable, "symbolicName")
      if name == driverVariableName:
        AdminConfig.remove(variable)
        break


print "\nSaving configuration changes"
AdminConfig.save()

if dmgr:
  print "\nSynchronizing active nodes"
  sync = AdminControl.completeObjectName('WebSphere:*,type=DeploymentManager')
  AdminControl.invoke(sync, 'multiSync', 'true')

if errors:
  print "\nRational Integration Tester JDBC Provider uninstall completed with " + str(errors) + " errors"
else:
  print "\nRational Integration Tester JDBC Provider uninstall completed successfully"
